-- Set additional defaults, and load additional settings for fov zooming
function VehicleCamera_insteadof_loadFromXML(me, superFunc, xmlFile, key)
	if not superFunc(me, xmlFile, key) then
		return false
	end

	me.origFovY = me.fovY
	me.fovTarget = me.fovY

	if me.isInside and me.limit then
		me.fovMin = getXMLFloat(xmlFile, key .. "#fovMin") or 10
		me.fovMax = getXMLFloat(xmlFile, key .. "#fovMax") or 100

		local fovDeg = math.deg(me.fovY)
		if me.fovMin > me.fovMax or fovDeg < me.fovMin or fovDeg > me.fovMax then
			g_logManager:xmlWarning(me.vehicle.configFileName, "Invalid FOV configuration for camera '%s', ensure fovMin <= fov <= fovMax", key)

			return false
		end
	end

	return true
end

-- Apply the same methodology used for smooth outside camera translation to inside FOV adjustment
function VehicleCamera_after_zoomSmoothly(me, offset)
	local fovTarget = me.fovTarget

	if me.isInside then
		fovTarget = math.max(math.min(me.fovTarget + math.rad(offset*3), math.rad(me.fovMax)), math.rad(me.fovMin))
	end

	me.fovTarget = fovTarget
end

-- Include FOV settings in camera reset
function VehicleCamera_after_resetCamera(me)
	me.fovY = me.origFovY
	me.fovTarget = me.origFovY
end

-- Smooth FOV adjustment, similar to smooth translation changes, and trigger debug drawing
function VehicleCamera_after_update(me, dt)
	local target = me.fovTarget or me.fovY

	me.fovY = target + math.pow(0.99579, dt) * (me.fovY - target)
	setFovY(me.cameraNode, me.fovY)

	if me.isActivated and me.isInside and Input.isMouseButtonPressed(Input.MOUSE_BUTTON_MIDDLE) then
		--me.fovY = me.origFovY
		me.fovTarget = me.origFovY
	end

	me:debugDraw()
end

-- Clean up the console command on deactivate
function VehicleCamera_after_onDeactivate(me)
	if me.isInside then
		removeConsoleCommand("pfToggleInsideVehicleCameraDebug")
	end
end

-- Toggle the debug display on and off, monkey patched into VehicleCamera for easy access to self
function VehicleCamera:consoleCommandToggleDebug()
	self.debugMode = not self.debugMode
end

-- Draw the actual debug information
function VehicleCamera:debugDraw()
	if self.debugMode and self.isInside then
		setTextColor(1, 1, 0, 1)
		renderText(0.05, 0.9, 0.02, "[inside camera]")
		renderText(0.05, 0.88, 0.02, "minFov(" .. self.fovMin .. ")")
		renderText(0.05, 0.86, 0.02, "fovY(" .. MathUtil.round(math.deg(self.fovY), 2) .. ")")
		renderText(0.05, 0.84, 0.02, "maxFov(" .. self.fovMax .. ")")
	end
end

-- Register the console command only while the inside camera is active
function VehicleCamera_after_onActivate(me)
	if me.isInside then
		addConsoleCommand("pfToggleInsideVehicleCameraDebug", "Toggle debug mode for inside vehicle cameras", "consoleCommandToggleDebug", VehicleCamera)
	end
end

-- Functionality injections, somehow, loadFromXML fails when appended, so opting for an override here
function initMod()
	VehicleCamera.zoomSmoothly = Utils.appendedFunction(VehicleCamera.zoomSmoothly, VehicleCamera_after_zoomSmoothly)
	VehicleCamera.loadFromXML  = Utils.overwrittenFunction(VehicleCamera.loadFromXML, VehicleCamera_insteadof_loadFromXML)
	VehicleCamera.resetCamera  = Utils.appendedFunction(VehicleCamera.resetCamera, VehicleCamera_after_resetCamera)
	VehicleCamera.update 			 = Utils.appendedFunction(VehicleCamera.update, VehicleCamera_after_update)
	VehicleCamera.onActivate 	 = Utils.appendedFunction(VehicleCamera.onActivate, VehicleCamera_after_onActivate)
	VehicleCamera.onDeactivate = Utils.appendedFunction(VehicleCamera.onDeactivate, VehicleCamera_after_onDeactivate)
end

-- Kick things off
initMod()